import ddtank


def get_game_window_information() -> list:
    """
    获取所有游戏窗口信息
    :return: 由游戏窗口信息字典组成的列表
        platform: 代理平台
        service: 服务器
        name: 账号或备注
        index: 窗口编号
        hwnd: 游戏窗口句柄
    """
    windows_list, info_dict_list = [], []
    ddtank.win32gui.EnumWindows(lambda w, param: param.append(w), windows_list)
    pattern_for_36 = ddtank.re.compile(r'\[(.*)-(.*)-(.*)]\|(.*)\|(.*)\|(.*)\|(.*)\|(.*)')
    for window in windows_list:
        title = ddtank.win32gui.GetWindowText(window)
        re_rst = pattern_for_36.search(title)
        if re_rst:
            rst = re_rst.groups()
            info_dict = {'platform': rst[0], 'service': int(rst[1]), 'name': rst[2], 'index': int(rst[3]), 'hwnd': int(rst[5])}
            info_dict_list.append(info_dict)
    return info_dict_list


def get_game_window_handle(title_pattern: str = r'.*') -> list:
    """
    获取所有游戏窗口句柄
    :param: 窗口标题匹配模式字符串，默认所有
    :return: 由游戏窗口句柄组成的列表
    """
    def get_all_child_window(parent):
        l = []
        ddtank.win32gui.EnumChildWindows(
            parent, lambda hwnd, param: param.append(hwnd), l)
        return l

    windows_list, hwnd_list = [], []
    ddtank.win32gui.EnumWindows(lambda w, param: param.append(w), windows_list)
    pattern_for_36 = ddtank.re.compile(r'\[(.*)-(.*)-(.*)]\|(.*)\|(.*)\|(.*)\|(.*)\|(.*)')
    if not title_pattern:
        title_pattern = pattern_for_36
    else:
        title_pattern = ddtank.re.compile(title_pattern)
    for window in windows_list:
        title = ddtank.win32gui.GetWindowText(window)
        re_rst = title_pattern.search(title)
        if re_rst:
            parent_title = re_rst.group()
            parent_hwnd = ddtank.win32gui.FindWindow(0, parent_title)
            child_hwnd_list = get_all_child_window(parent_hwnd)
            for child_hwnd in child_hwnd_list:
                shape = ddtank.win32gui.GetWindowRect(child_hwnd)
                class_name = ddtank.win32gui.GetClassName(child_hwnd)
                if class_name == 'MacromediaFlashPlayerActiveX':
                    hwnd_list.append(child_hwnd)
    return hwnd_list





import ddtank
from ddtank.status import Status

class DDTask:
    status_list = []

    def __init__(self):
        pass

    def __add__(self, other: Status) -> bool:
        """
        添加一个角色到任务队列
        :param other: 被添加的角色
        :return: 返回布尔类型变量，代表操作是否成功
        """
        self.status_list += [other]
        return True

    def __sub__(self, other: Status) -> bool:
        """
        从任务队列删除一个角色
        :param other: 被删除的角色
        :return: 返回布尔类型变量，代表操作是否成功
        """
        if other in self.status_list:
            self.status_list.remove(other)
            return True
        return False

    def __repr__(self) -> str:
        return str(self.status_list)

    def perform(self):
        """
        执行任务队列中所有角色的任务
        :return: 无返回值
        """
        for status in self.status_list:
            ddtank.Thread(target=status.task).start()



import re
import numpy as np


class BaseRecLabelDecode(object):
    """ Convert between text-label and text-index """

    def __init__(self, character_dict_path=None, use_space_char=False):
        self.beg_str = "sos"
        self.end_str = "eos"
        self.reverse = False
        self.character_str = []

        if character_dict_path is None:
            self.character_str = "0123456789abcdefghijklmnopqrstuvwxyz"
            dict_character = list(self.character_str)
        else:
            with open(character_dict_path, "rb") as fin:
                lines = fin.readlines()
                for line in lines:
                    line = line.decode('utf-8').strip("\n").strip("\r\n")
                    self.character_str.append(line)
            if use_space_char:
                self.character_str.append(" ")
            dict_character = list(self.character_str)
            if 'arabic' in character_dict_path:
                self.reverse = True

        dict_character = self.add_special_char(dict_character)
        self.dict = {}
        for i, char in enumerate(dict_character):
            self.dict[char] = i
        self.character = dict_character

    def pred_reverse(self, pred):
        pred_re = []
        c_current = ''
        for c in pred:
            if not bool(re.search('[a-zA-Z0-9 :*./%+-]', c)):
                if c_current != '':
                    pred_re.append(c_current)
                pred_re.append(c)
                c_current = ''
            else:
                c_current += c
        if c_current != '':
            pred_re.append(c_current)

        return ''.join(pred_re[::-1])

    def add_special_char(self, dict_character):
        return dict_character

    def decode(self, text_index, text_prob=None, is_remove_duplicate=False):
        """ convert text-index into text-label. """
        result_list = []
        ignored_tokens = self.get_ignored_tokens()
        batch_size = len(text_index)
        for batch_idx in range(batch_size):
            selection = np.ones(len(text_index[batch_idx]), dtype=bool)
            if is_remove_duplicate:
                selection[1:] = text_index[batch_idx][1:] != text_index[
                                                                 batch_idx][:-1]
            for ignored_token in ignored_tokens:
                selection &= text_index[batch_idx] != ignored_token

            char_list = [
                self.character[text_id]
                for text_id in text_index[batch_idx][selection]
            ]
            if text_prob is not None:
                conf_list = text_prob[batch_idx][selection]
            else:
                conf_list = [1] * len(selection)
            if len(conf_list) == 0:
                conf_list = [0]

            text = ''.join(char_list)

            if self.reverse:  # for arabic rec
                text = self.pred_reverse(text)

            result_list.append((text, np.mean(conf_list).tolist()))
        return result_list

    def get_ignored_tokens(self):
        return [0]  # for ctc blank


class CTCLabelDecode(BaseRecLabelDecode):
    """ Convert between text-label and text-index """

    def __init__(self, character_dict_path=None, use_space_char=False,
                 **kwargs):
        super(CTCLabelDecode, self).__init__(character_dict_path,
                                             use_space_char)

    def __call__(self, preds: np.ndarray, label=None, *args, **kwargs):
        if isinstance(preds, tuple) or isinstance(preds, list):
            preds = preds[-1]
        preds_idx = preds.argmax(axis=2)
        preds_prob = preds.max(axis=2)
        text = self.decode(preds_idx, preds_prob, is_remove_duplicate=True)
        if label is None:
            return text
        label = self.decode(label)
        return text, label

    def add_special_char(self, dict_character):
        dict_character = ['blank'] + dict_character
        return dict_character


def build_postprocess(character_dict_path, algorithm="CTC"):
    if algorithm == "CTC":
        return CTCLabelDecode(character_dict_path)
    else:
        raise ValueError(f"Unexpected value algorithm={algorithm}.")


__version__ = '0.0.9'


import win32api
import win32con
import win32gui
import win32ui
import time
import string
import cv2
import numpy as np
import re
import os

from ctypes import windll
from threading import Thread

from .dl_script import load_onnx, build_preprocess, build_postprocess

VkKeyScanA = windll.user32.VkKeyScanA
